<?php
/**
 * @license   http://www.gnu.org/licenses/gpl.html GPL Version 3
 * @author    Volker Theile <volker.theile@openmediavault.org>
 * @author    OpenMediaVault Plugin Developers <plugins@omv-extras.org>
 * @copyright Copyright (c) 2009-2013 Volker Theile
 * @copyright Copyright (c) 2013-2014 OpenMediaVault Plugin Developers
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
require_once("openmediavault/object.inc");
require_once("openmediavault/config.inc");
require_once("openmediavault/error.inc");
require_once("openmediavault/util.inc");
require_once("openmediavault/rpcservice.inc");
require_once("openmediavault/notify.inc");

class OMVRpcServiceMiniDlna extends OMVRpcServiceAbstract {
    /**
     * Get the RPC service name.
     */
    public function getName() {
        return "MiniDLNA";
    }

    /**
     * Initialize the RPC service.
     */
    public function initialize() {
        $this->registerMethod("getSettings");
        $this->registerMethod("setSettings");
        $this->registerMethod("getShareList");
        $this->registerMethod("getShare");
        $this->registerMethod("setShare");
        $this->registerMethod("deleteShare");
        $this->registerMethod("doRescan");
    }

    /**
     * Get the settings configuration object.
     * @param params The method parameters.
     * @param context The context of the caller.
     * @return The requested configuration object.
     * @throw E_CONFIG_GET_OBJECT_FAILED
     */
    public function getSettings($params, $context) {
        global $xmlConfig;
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
              "role" => OMV_ROLE_ADMINISTRATOR
          ));
        // Get the configuration object.
        $xpath = "//services/minidlna";
        $object = $xmlConfig->get($xpath);
        if(is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED,
              $xpath);
        }

        // Modify the result object.
        $object['enable'] = boolval($object['enable']);
        $object['port']   = intval($object['port']);
        $object['strict'] = intval($object['strict']);
        $object['tivo']   = intval($object['tivo']);

        // Remove useless fields from the result object.
        unset($object['shares']);
        return $object;
    }

    /**
     * Set the settings configuration object.
     * @param params The method parameters.
     * @param context The context of the caller.
     * @return The stored configuration object.
     * @throw E_CONFIG_GET_OBJECT_FAILED
     * @throw E_CONFIG_SET_OBJECT_FAILED
     */
    public function setSettings($params, $context) {
        global $xmlConfig;
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
            "role" => OMV_ROLE_ADMINISTRATOR
        ));
        // Validate the parameters of the RPC service method.
        $this->validateMethodParams($params, '{
            "type"        : "object",
            "properties"  : {
                "enable" : { "type" : "boolean" },
                "name"   : { "type" : "string"},
                "port"   : { "type" : "integer", "minimum" : 1, "maximum" : 65535 },
                "strict" : { "type" : "boolean"},
                "tivo"   : { "type" : "boolean"}
            }
        }');

        // Get the existing configuration object.
        $xpath = "//services/minidlna";
        $oldObject = $xmlConfig->get($xpath);
        if(is_null($oldObject))
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);

        // Update the configuration object.
        $object = array(
            "enable" => array_boolval($params, 'enable'),
            "name"   => $params['name'],
            "port"   => $params['port'],
            "strict" => array_boolval($params, 'strict'),
            "tivo"   => array_boolval($params, 'tivo'),
            "shares" => isset($oldObject['shares']) ? $oldObject['shares'] : array(),
        );
        if(FALSE === $xmlConfig->replace("//services/minidlna", $object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);
        }
        // Notify configuration changes.
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(OMV_NOTIFY_MODIFY,
          "org.openmediavault.services.minidlna", $object);
        // Return the configuration object.
        return $object;
    }

    /**
     * Get list of share configuration objects.
     * @param params An array containing the following fields:
     *   \em start The index where to start.
     *   \em limit The number of objects to process.
     *   \em sortfield The name of the column used to sort.
     *   \em sortdir The sort direction, ASC or DESC.
     * @param context The context of the caller.
     * @return An array containing the requested objects. The field \em total
     *   contains the total number of objects, \em data contains the object
     *   array. An exception will be thrown in case of an error.
     * @throw E_CONFIG_GET_OBJECT_FAILED
     */
    public function getShareList($params, $context) {
        global $xmlConfig;
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
              "role" => OMV_ROLE_ADMINISTRATOR
          ));
        // Validate the parameters of the RPC service method.
        $this->validateMethodParams($params, '{
            "type"       : "object",
            "properties" : {
                "start"     : { "type" : "integer" },
                "limit"     : { '.$GLOBALS['OMV_JSONSCHEMA_COUNTFIELD'].' },
                "sortfield" : { '.$GLOBALS['OMV_JSONSCHEMA_SORTFIELD'].' },
                "sortdir"   : { '.$GLOBALS['OMV_JSONSCHEMA_SORTDIR'].' }
            }
        }');
        // Get the configuration object.
        $xpath = "//services/minidlna/shares/share";
        $objects = $xmlConfig->getList($xpath);
        if(is_null($objects)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }
        // Add additional share informations.
        foreach($objects as $objectk => &$objectv) {
            $objectv['sharedfoldername'] = gettext("n/a");
            // Get the shared folder configuration object.
            $sfObject = $xmlConfig->get("//system/shares/sharedfolder[".
              "uuid='{$objectv['sharedfolderref']}']");
            if(!is_null($sfObject) && !empty($sfObject)) {
                $objectv['sharedfoldername'] = $sfObject['name'];
            }
        }
        // Filter the result.
        return $this->applyFilter($objects, $params['start'],
          $params['limit'], $params['sortfield'], $params['sortdir']);
    }

    /**
     * Get a share config object.
     * @param params An array containing the following fields:
     *   \em uuid The UUID of the configuration object.
     * @param context The context of the caller.
     * @return The requested configuration object.
     * @throw E_CONFIG_GET_OBJECT_FAILED
     */
    public function getShare($params, $context) {
        global $xmlConfig;
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
              "role" => OMV_ROLE_ADMINISTRATOR
          ));
        // Validate the parameters of the RPC service method.
        $this->validateMethodParams($params, '{
            "type"       : "object",
            "properties" : {
                "uuid" : { '.$GLOBALS['OMV_JSONSCHEMA_UUID'].' }
            }
        }');
        // Get the configuration object.
        $xpath = sprintf("//services/minidlna/shares/share[uuid='%s']",
          $params['uuid']);
        $object = $xmlConfig->get($xpath);
        if(is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }
        return $object;
    }

    /**
     * Set a share config object.
     * @param params The method parameters.
     * @param context The context of the caller.
     * @return The stored configuration object.
     * @throw E_CONFIG_OBJECT_UNIQUENESS
     * @throw E_CONFIG_GET_OBJECT_FAILED
     * @throw E_CONFIG_SET_OBJECT_FAILED
     */
    public function setShare($params, $context) {
        global $xmlConfig;
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
              "role" => OMV_ROLE_ADMINISTRATOR
          ));
        // Validate the parameters of the RPC service method.
        $this->validateMethodParams($params, '{
            "type"       : "object",
            "properties" : {
                "uuid"            : { '.$GLOBALS['OMV_JSONSCHEMA_UUID_UNDEFINED'].' },
                "sharedfolderref" : { '.$GLOBALS['OMV_JSONSCHEMA_UUID'].' },
                "mtype"           : { "type" : "string" }
            }
        }');
        // Prepare the configuration object.
        $object = array(
            "uuid"            => ($params['uuid'] == $GLOBALS['OMV_UUID_UNDEFINED']) ?
                OMVUtil::uuid() : $params['uuid'],
            "sharedfolderref" => $params['sharedfolderref'],
            "mtype"           => $params['mtype']
        );
        // Set the configuration object.
        $success = FALSE;
        if($params['uuid'] == $GLOBALS['OMV_UUID_UNDEFINED']) {
            // Check uniqueness. A shared folder can only be configured once
            // for MiniDlna.
            if(TRUE === $xmlConfig->exists("//services/minidlna/shares/share[" .
              "sharedfolderref='{$params['sharedfolderref']}']")) {
                throw new OMVException(OMVErrorMsg::E_CONFIG_OBJECT_UNIQUENESS,
                  gettext("The shared folder is already used by another share."));
            }
            // Get the shared folder object.
            $xpath = sprintf("//system/shares/sharedfolder[uuid='%s']",
              $params['sharedfolderref']);
            $sfObject = $xmlConfig->get($xpath);
            if(is_null($sfObject)) {
                throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED,
                  $xpath);
            }
            // Get the absolute shared folder path.
            $sfPath = OMVRpc::exec("ShareMgmt", "getPath", array(
              "uuid" => $params['sharedfolderref']), $context);
            // Append the configuration object.
            $success = $xmlConfig->set("//services/minidlna/shares",
              array("share" => $object));
        } else {
            // Update the existing configuration object.
            $success = $xmlConfig->replace("//services/minidlna/shares/share[" .
              "uuid='{$params['uuid']}']", $object);
        }
        if(FALSE === $success) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);
        }
        // Notify configuration changes.
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(($params['uuid'] == $GLOBALS['OMV_UUID_UNDEFINED']) ?
          OMV_NOTIFY_CREATE : OMV_NOTIFY_MODIFY,
          "org.openmediavault.services.minidlna.shares.share", $object);
        // Return the configuration object.
        return $object;
    }

    /**
     * Delete a share configuration object.
     * @param params An array containing the following fields:
     *   \em uuid The UUID of the configuration object.
     * @param context The context of the caller.
     * @return The deleted configuration object.
     */
    public function deleteShare($params, $context) {
        // Validate the RPC caller context.
        $this->validateMethodContext($context, array(
              "role" => OMV_ROLE_ADMINISTRATOR
          ));
        // Validate the parameters of the RPC service method.
        $this->validateMethodParams($params, '{
            "type"       : "object",
            "properties" : {
                "uuid" : { '.$GLOBALS['OMV_JSONSCHEMA_UUID'].' }
            }
        }');
        // Delete the configuration object.
        $xpath = sprintf("//services/minidlna/shares/share[uuid='%s']",
          $params['uuid']);
        $object = $this->deleteConfigObjectByPath($xpath,
          "org.openmediavault.services.minidlna.shares.share");
        // Return the deleted configuration object.
        return $object;
    }

    public function doRescan($params, $context) {
        global $xmlConfig;

        // Validate the RPC caller context.
        $this->validateMethodContext($context, array( "role" => OMV_ROLE_ADMINISTRATOR ));

        $xpath = "//services/minidlna";
        $object = $xmlConfig->get($xpath);
        if(is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }

        if(FALSE === boolval($object['enable']))
            return;

        $cmd = "sudo /etc/init.d/minidlna force-reload 2>&1";
        if(0 !== $this->exec($cmd, $output)) {
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED, $cmd, implode("\n", $output));
        }
    }
}

// Register the RPC service.
$rpcServiceMgr = &OMVRpcServiceMgr::getInstance();
$rpcServiceMgr->registerService(new OMVRpcServiceMiniDlna());
?>
